%averhst - Avereging histogram among +-StepX neighbors
function [HstV,MBinsZ,Outside]=averhstl(HstA,MBinsZ,StepA,AverMode,AverOutside);

%Outputs:
%HstV - histogram averaged among StepA nearest neighbors
%Outside - number of bins which was added at every histogram side for outside averaging

%Inputs:
%HstA - Initial histogram vector.
%StepA - Number of nearest +- neighbors to average.
%AverMode - mode of averaging : running average <1>, triangular shape running average <2>, Averaging with custom coefficients <3>
%AverOutside - Option for the Range of averaging - do not average outside <1>, average boarding bins with outside as 0 <2>, or make circular averaging (join last bin with the first bin)

Outside=0;
switch AverMode
case {1 2}	%mean, smoothing averaging
	Outside=StepA;	
case 3	%Averaging with custom coefficients
	Outside=length(StepA)-1;
case 4	%Averaging with +- Standard deviation
	% Calculate bins coefficients based on standard deviation
	BinWidth=MBinsZ(2)-MBinsZ(1);
	AveragingBins=(-0.5*BinWidth):BinWidth:(4*StepA+0.5*BinWidth);	% Bins up to 3 standard deviations
	StepA=diff(normcdf(AveragingBins,0,StepA));
	Outside=length(StepA)-1;
end

%Adding additional bins, if necessary
switch AverOutside	%Range of averaging
case 1	%'Do not average boarding bins'
case 2	%Averaging boarding bins with outside as 0
	if Outside~=0	%number of bins outside the array
		s=size(HstA,1);
		HstA=[zeros(s,Outside), HstA, zeros(s,Outside)];
	end
case 3	%'Circular averaging'
	if Outside~=0	%number of bins outside the array
		s=size(HstA,1);
		HstA=[HstA(:,(end-Outside+1):end), HstA, HstA(:,1:Outside)];	%Adding bins to make a circle
	end
end

lA=size(HstA,2);
TotalValue=sum(HstA,2);	% Total initial sum value for each histogram
for n=1:size(HstA,1)	% Cycle through different histograms
	for i=1:lA	% Cycle through the bins of the current histogram
		StepAm=min([Outside,i-1,lA-i]);	%minimal symmetrical averaging
		switch AverMode
		case 1	%mean
			HstV(n,i)=mean(HstA(n,i-StepAm:i+StepAm));
			%'mean'
		case 2	%smoothing averaging
			%'smoothing averaging'
			if StepAm>0	%smoothing averaging
				SmVect=[];
				SmVect(1:(StepAm+1))=1:(StepAm+1);	%1st part of smoothing vector
				SmVect((StepAm+2):(2*StepAm+1))=StepAm-(0:(StepAm-1));	%2nd part of smoothing vector
				HstV(n,i)=sum(HstA(n,i-StepAm:i+StepAm).*SmVect)/(sum(SmVect));
			else
				HstV(n,i)=HstA(n,i);
			end
		case 3	%Averaging with custom coefficients
			%'smoothing averaging'
			if StepAm>0	%smoothing averaging
				SmVect=[];
				SmVect((StepAm+1):-1:1)=StepA(1:(StepAm+1));	%1st part of smoothing vector
				SmVect((StepAm+2):(2*StepAm+1))=SmVect(StepAm:-1:1);	%2nd part of smoothing vector
				HstV(n,i)=sum(HstA(n,i-StepAm:i+StepAm).*SmVect);
			else
				HstV(n,i)=HstA(n,i);
			end
		case 4	% Averaging with +- Standard deviation
			%'smoothing averaging'
			if StepAm>0	%smoothing averaging
				SmVect=[];
				SmVect((StepAm+1):-1:1)=StepA(1:(StepAm+1));	%1st part of smoothing vector
				SmVect((StepAm+2):(2*StepAm+1))=SmVect(StepAm:-1:1);	%2nd part of smoothing vector
				HstV(n,i)=sum(HstA(n,i-StepAm:i+StepAm).*SmVect);
			else
				HstV(n,i)=HstA(n,i);
			end
		end
	end	% Cycle through the bins of the current histogram
	HstV(n,:)=TotalValue(n)*HstV(n,:)/sum(HstV(n,:));	% Normalize the histogram to preserve an initial total value
end	% Cycle through different histograms

%Removing additional bins, if necessary
switch AverOutside	%Range of averaging
case 1	%'Do not average boarding bins'
case 2	%Averaging boarding bins with outside as 0
	Stepout=(max(MBinsZ)-min(MBinsZ))/(length(MBinsZ)-1);
	MBinsZ=[(min(MBinsZ)-Outside*Stepout):Stepout:(max(MBinsZ)+Outside*Stepout)]';	%Additional bins
case 3	%'Circular averaging'
	if Outside~=0	%number of bins outside the array
		HstV=HstV(:,(1+Outside):(end-Outside));	%Removing bins to make a circle
	end
end


